@@ -16,7 +16,7 @@ from rich.theme import Theme
 from natsort import natsorted, ns
 import regex
 
-VERSION = "1.5.7"
+VERSION = "1.6.0"
 MESSAGE_INDENT = "    "
 UNICODE_BOM = "\ufeff"
 NO_BREAK_SPACE = "\u00a0"
@@ -44,7 +44,8 @@ RICH_THEME = Theme({
 	"path": "bright_blue",
 	"url": "bright_blue",
 	"text": "bright_blue",
-	"bash": "bright_blue"
+	"bash": "bright_blue",
+	"css": "bright_blue"
 })
 
 class SeException(Exception):
