@@ -4,6 +4,8 @@ This module implements the `se hyphenate` command.
 
 import argparse
 
+from rich.console import Console
+
 import se
 import se.typography
 
@@ -20,13 +22,15 @@ def hyphenate() -> int:
 	parser.add_argument("targets", metavar="TARGET", nargs="+", help="an XHTML file, or a directory containing XHTML files")
 	args = parser.parse_args()
 
+	console = Console(highlight=False, theme=se.RICH_THEME, force_terminal=se.is_called_from_parallel()) # Syntax highlighting will do weird things when printing paths; force_terminal prints colors when called from GNU Parallel
+
 	for filename in se.get_target_filenames(args.targets, (".xhtml",)):
 		if args.verbose:
-			print(f"Processing {filename} ...", end="", flush=True)
+			console.print(f"Processing [path][link=file://{filename}]{filename}[/][/] ...", end="")
 
 		se.typography.hyphenate_file(filename, args.language, args.ignore_h_tags)
 
 		if args.verbose:
-			print(" OK")
+			console.print(" OK")
 
 	return 0
