@@ -5,6 +5,8 @@ The class exposes some helpful functions like css_select() and xpath().
 """
 
 from typing import Dict, List, Union
+import unicodedata
+
 import regex
 from lxml import cssselect, etree
 import se
@@ -65,6 +67,18 @@ class EasyXmlTree:
 
 		return result
 
+	def tostring(self) -> str:
+		"""
+		Serialize the tree to a string.
+		"""
+
+		xml = """<?xml version="1.0" encoding="utf-8"?>\n""" + etree.tostring(self.etree, encoding="unicode") + "\n"
+
+		# Normalize unicode characters
+		xml = unicodedata.normalize("NFC", xml)
+
+		return xml
+
 class EasyXhtmlTree(EasyXmlTree):
 	"""
 	Wrapper for the XHTML namespace.
@@ -76,6 +90,17 @@ class EasyXhtmlTree(EasyXmlTree):
 
 		EasyXmlTree.__init__(self, xml_string.replace(" xmlns=\"http://www.w3.org/1999/xhtml\"", ""))
 
+	def tostring(self) -> str:
+		"""
+		Serialize the tree to a string.
+		"""
+
+		xml = EasyXmlTree.tostring(self)
+
+		xml = xml.replace("<html", "<html xmlns=\"http://www.w3.org/1999/xhtml\"")
+
+		return xml
+
 class EasySvgTree(EasyXmlTree):
 	"""
 	Wrapper for the SVG namespace.
@@ -87,9 +112,20 @@ class EasySvgTree(EasyXmlTree):
 
 		EasyXmlTree.__init__(self, xml_string.replace(" xmlns=\"http://www.w3.org/2000/svg\"", ""))
 
+	def tostring(self) -> str:
+		"""
+		Serialize the tree to a string.
+		"""
+
+		xml = EasyXmlTree.tostring(self)
+
+		xml = xml.replace("<svg", "<svg xmlns=\"http://www.w3.org/2000/svg\"")
+
+		return xml
+
 class EasyOpfTree(EasyXmlTree):
 	"""
-	Wrapper for the SVG namespace.
+	Wrapper for the OPF namespace.
 	"""
 
 	def __init__(self, xml_string: str):
@@ -98,6 +134,17 @@ class EasyOpfTree(EasyXmlTree):
 
 		EasyXmlTree.__init__(self, xml_string.replace(" xmlns=\"http://www.idpf.org/2007/opf\"", ""))
 
+	def tostring(self) -> str:
+		"""
+		Serialize the tree to a string.
+		"""
+
+		xml = EasyXmlTree.tostring(self)
+
+		xml = xml.replace("<package", "<package xmlns=\"http://www.idpf.org/2007/opf\"")
+
+		return xml
+
 class EasyXmlElement:
 	"""
 	Represents an lxml element.
@@ -141,7 +188,6 @@ class EasyXmlElement:
 		Return the value of an attribute on this element.
 		"""
 
-
 		attribute = attribute.replace("epub:", "{http://www.idpf.org/2007/ops}")
 
 		return self.lxml_element.get(attribute)
@@ -212,6 +258,53 @@ class EasyXmlElement:
 
 		parent.remove(self.lxml_element)
 
+	def unwrap(self) -> None:
+		"""
+		Remove the element's wrapping tag and replace it with the element's contents.
+		"""
+
+		# In lxml, there are no "text nodes" like in a classic DOM. There are only element nodes.
+		# An element has a `.text` property which is the child text UP TO THE FIRST CHILD ELEMENT.
+		# An element's `.tail` property contains text *after* the element, up to its first element sibling.
+
+		parent = self.lxml_element.getparent()
+
+		children = self.lxml_element.getchildren()
+
+		children.reverse()
+
+		# This will *move* each child element node to *after* the current element.
+		# Since any following text is stored in the child element's .tail, this will *also*
+		# move that text.
+		for child in children:
+			self.lxml_element.addnext(child)
+
+		# Now we've moved all child elements and the text following them. But what if there's
+		# text *before* any child elements? That is stored in the .text property.
+		if self.lxml_element.text:
+			prev = self.lxml_element.getprevious()
+			if prev is None:
+				if parent.text:
+					parent.text = parent.text + self.lxml_element.text
+				else:
+					parent.text = self.lxml_element.text
+			else:
+				if prev.tail:
+					prev.tail = prev.tail + self.lxml_element.text
+				else:
+					prev.tail = self.lxml_element.text
+
+		# This calls the EasyXmlTree.remove() function, not an lxml function
+		self.remove()
+
+	def replace_with(self, node) -> None:
+		"""
+		Remove this node and replace it with the passed node
+		"""
+
+		self.lxml_element.addnext(node.lxml_element)
+		self.remove()
+
 	@property
 	def text(self) -> str:
 		"""
