@@ -4,6 +4,7 @@ Defines functions useful for formatting code or text according to SE standards,
 several text-level statistics like reading ease, and for adding semantics.
 """
 
+from copy import deepcopy
 import html.entities
 import math
 import string
@@ -13,10 +14,9 @@ from pathlib import Path
 import regex
 import roman
 import tinycss2
-from bs4 import BeautifulSoup, NavigableString, Tag
 from lxml import etree
 from titlecase import titlecase as pip_titlecase
-#from se.vendor.titlecase import titlecase as pip_titlecase
+from se.easy_xml import EasyXhtmlTree
 
 import se
 
@@ -86,20 +86,24 @@ def semanticate(xhtml: str) -> str:
 	xhtml = regex.sub(r"\b(?<!\<abbr\>)([Cc])f\.", r"<abbr>\1f.</abbr>", xhtml)
 	xhtml = regex.sub(r"\b(?<!\<abbr\>)p\.([\s0-9])", r"<abbr>p.</abbr>\1", xhtml)
 	xhtml = regex.sub(r"\b(?<!\<abbr\>)ed\.", r"<abbr>ed.</abbr>", xhtml)
-	xhtml = regex.sub(r"""(?<!\<abbr class="initialism( eoc)?"\>)([Ii])\.e\.""", r"""<abbr class="initialism">\1.e.</abbr>""", xhtml)
-	xhtml = regex.sub(r"""(?<!\<abbr class="initialism( eoc)?"\>)([Ee])\.g\.""", r"""<abbr class="initialism">\1.g.</abbr>""", xhtml)
-	xhtml = regex.sub(r"""(?<!\<abbr class="initialism( eoc)?"\>)\bN\.?B\.\b""", r"""<abbr class="initialism">N.B.</abbr>""", xhtml)
 	xhtml = regex.sub(r"(?<!\<abbr\>)(Jan\.|Feb\.|Mar\.|Apr\.|Jun\.|Jul\.|Aug\.|Sep\.|Sept\.|Oct\.|Nov\.|Dec\.)", r"<abbr>\1</abbr>", xhtml)
 	xhtml = regex.sub(r"(?<!\<abbr\>)No\.(\s+[0-9]+)", r"<abbr>No.</abbr>\1", xhtml)
+	xhtml = regex.sub(r"\b(?<!\<abbr\>)([Vv])s\.", r"<abbr>\1s.</abbr>", xhtml)
+	# python allows a variable lookbehind with the ( eoc)?; HOWEVER, it counts it as the
+	#	first capture group, so if there are one or more capture groups in the regex itself,
+	#	be sure to start at 2 when specifying the group(s) in the replacement spec.
+	xhtml = regex.sub(r"""(?<!\<abbr class="initialism( eoc)?"\>)([Ii])\.e\.""", r"""<abbr class="initialism">\2.e.</abbr>""", xhtml)
+	xhtml = regex.sub(r"""(?<!\<abbr class="initialism( eoc)?"\>)([Ee])\.g\.""", r"""<abbr class="initialism">\2.g.</abbr>""", xhtml)
+	xhtml = regex.sub(r"""(?<!\<abbr class="initialism( eoc)?"\>)\bN\.?B\.\b""", r"""<abbr class="initialism">N.B.</abbr>""", xhtml)
 	xhtml = regex.sub(r"""(?<!\<abbr class="degree( eoc)?"\>)Ph\.?\s*D\.?""", r"""<abbr class="degree">Ph. D.</abbr>""", xhtml)
 	xhtml = regex.sub(r"""(?<!\<abbr class="initialism( eoc)?"\>)I\.?O\.?U\.?\b""", r"""<abbr class="initialism">I.O.U.</abbr>""", xhtml)
 	xhtml = regex.sub(r"""(\s)(?<!\<abbr class="era( eoc)?"\>)A\.?D""", r"""\1<abbr class="era">AD</abbr>""", xhtml)
 	xhtml = regex.sub(r"""(\s)(?<!\<abbr class="era( eoc)?"\>)B\.?C""", r"""\1<abbr class="era">BC</abbr>""", xhtml)
 	xhtml = regex.sub(r"""(?<!\<abbr class="time( eoc)?"\>)([ap])\.\s?m\.""", r"""<abbr class="time">\2.m.</abbr>""", xhtml)
-	xhtml = regex.sub(r"\b(?<!\<abbr\>)([Vv])s\.", r"<abbr>\1s.</abbr>", xhtml)
 	xhtml = regex.sub(r"""(?<!\<abbr class="name( eoc)?"\>)Thos\.""", r"""<abbr class="name">Thos.</abbr>""", xhtml)
 	xhtml = regex.sub(r"""(?<!\<abbr class="name( eoc)?"\>)Jas\.""", r"""<abbr class="name">Jas.</abbr>""", xhtml)
 	xhtml = regex.sub(r"""(?<!\<abbr class="name (eoc)?"\>)Chas\.""", r"""<abbr class="name">Chas.</abbr>""", xhtml)
+	xhtml = regex.sub(r"""(?<!\<abbr class="initialism( eoc)?"\>)\b([1-4]D)\b""", r"""<abbr class="initialism">\2</abbr>""", xhtml)
 
 	# Wrap £sd shorthand
 	xhtml = regex.sub(r"([0-9½¼⅙⅚⅛⅜⅝]+)([sd⅞]\.)", r"\1<abbr>\2</abbr>", xhtml)
@@ -124,6 +128,12 @@ def semanticate(xhtml: str) -> str:
 	# Get Roman numerals that are X or V and single characters.  We can't do I for obvious reasons.
 	xhtml = regex.sub(r"""([^\p{Letter}>\"])([vxVX])(\b|st\b|nd\b|rd\b|th\b)""", r"""\1<span epub:type="z3998:roman">\2</span>\3""", xhtml)
 
+	# We can assume a lowercase i is always a Roman numeral
+	xhtml = regex.sub(r"""([^\p{Letter}>\"])(i)(\b|st\b)""", r"""\1<span epub:type="z3998:roman">\2</span>\3""", xhtml)
+
+	# Fix X-ray
+	xhtml = regex.sub(r"""<span epub:type="z3998:roman">([Xx])</span>-ray""", r"""\1-ray""", xhtml)
+
 	# Add abbrevations around some SI measurements
 	xhtml = regex.sub(r"([0-9]+)\s*([cmk][mgl])\b", fr"\1{se.NO_BREAK_SPACE}<abbr>\2</abbr>", xhtml)
 
@@ -599,7 +609,6 @@ def format_xml(xml: str) -> str:
 	except Exception as ex:
 		raise se.InvalidXmlException(f"Couldn’t parse XML file. Exception: {ex}")
 
-
 	# Pull out the doctype if there is one, as etree seems to eat it
 	doctypes = regex.search(r"<!doctype[^>]+?>", xml, flags=regex.IGNORECASE)
 
@@ -1142,7 +1151,7 @@ def namespace_to_class(selector: str) -> str:
 	A string representing the selector with namespaces replaced by classes
 	"""
 
-	# First, remove periods from epub:type.  We can't remove periods in the entire selector because there might be class selectors involved
+	# First, remove periods from epub:type.	 We can't remove periods in the entire selector because there might be class selectors involved
 	epub_type = regex.search(r"\"[^\"]+?\"", selector).group()
 	if epub_type:
 		selector = selector.replace(epub_type, epub_type.replace(".", "-"))
@@ -1203,7 +1212,7 @@ def simplify_css(css: str) -> str:
 	return css
 
 
-def generate_title(xhtml: str) -> str:
+def generate_title(xhtml) -> str:
 	"""
 	Generate the value for the <title> tag of a string of XHTML, based on the rules in the SE manual.
 
@@ -1214,59 +1223,93 @@ def generate_title(xhtml: str) -> str:
 	A string representing the title for the document
 	"""
 
-	soup = BeautifulSoup(xhtml, "lxml")
+	if isinstance(xhtml, str):
+		dom = EasyXhtmlTree(xhtml)
+	else:
+		dom = deepcopy(xhtml)
+
 	title = ""
 
-	h_elements = soup.select("h1:first-of-type,h2:first-of-type,h3:first-of-type,h4:first-of-type,h5:first-of-type,h6:first-of-type")
-
-	if h_elements:
-		h_element = h_elements[0]
+	# Do we have an hgroup element to process?
+	# Only match hgroups that do not have a ancestor containing an h# or header (which presumably contains an h# element). Note
+	# how we exclude <header>s that contain an <hgroup> otherwise we would match ourselves!
+	hgroup_elements = dom.xpath("//hgroup[not(ancestor::*[./*[re:test(name(), '^h[1-6]$') or (name() = 'header' and not(./hgroup))]])]")
+	if hgroup_elements:
+		hgroup_element = hgroup_elements[0]
 
 		# Strip any endnote references first
-		for endnote_tag in h_element.select("[id^='noteref']"):
-			endnote_tag.decompose()
+		for node in hgroup_element.xpath("//*[contains(@epub:type, 'noteref')]"):
+			node.remove()
 
-		# semos://1.0.0/5.3.2.2
-		# Header is just a Roman numeral
-		if h_element.has_attr("epub:type") and "z3998:roman" in h_element["epub:type"]:
-			title = f"Chapter {roman.fromRoman(h_element.text.upper())}"
+		closest_parent_sections = hgroup_element.xpath("./ancestor::*[name() = 'section' or name() = 'article'][1]")
 
-		# Otherwise, iterate over the h# children to determine how we should generate the title
+		if closest_parent_sections:
+			closest_parent_section = closest_parent_sections[0]
 		else:
-			for h_child in h_element.contents:
-				if isinstance(h_child, Tag) and h_child.name == "span":
-					if h_child.has_attr("epub:type"):
-						if "z3998:roman" in h_child["epub:type"]:
-							title = f"Chapter {roman.fromRoman(h_child.text.upper())}"
+			raise se.InvalidSeEbookException("No [xhtml]<section>[/] or [xhtml]<article>[/] element for [xhtml]<hgroup>[/]")
 
-						if "subtitle" in h_child["epub:type"]:
-							title += f": {h_child.text}"
-					else:
-						if len(h_child.contents) > 1:
-							for span_child in h_child.contents:
-								if isinstance(span_child, Tag) and span_child.name == "span":
-									if span_child.has_attr("epub:type") and "z3998:roman" in span_child["epub:type"]:
-										title += str(roman.fromRoman(span_child.text.upper()))
-									else:
-										title += span_child.text
-								elif isinstance(span_child, NavigableString) and not span_child.isspace():
-									title += span_child
-						else:
-							title = h_child.text
-				elif isinstance(h_child, Tag) and (h_child.name == "abbr" or h_child.name == "i"):
-					title += h_child.text
-				elif isinstance(h_child, NavigableString) and not h_child.isspace():
-					title += h_child
+		# If the closest parent <section> or <article> is a part, division, or volume, then keep all <hgroup> children
+		if closest_parent_section.attribute("epub:type") and ("part" not in closest_parent_section.attribute("epub:type") and "division" not in closest_parent_section.attribute("epub:type") and "volume" not in closest_parent_section.attribute("epub:type")):
+			# Else, if the closest parent <section> or <article> is a halftitlepage, then discard <hgroup> subtitles
+			if closest_parent_section.attribute("epub:type") and "halftitlepage" in closest_parent_section.attribute("epub:type"):
+				for node in hgroup_element.xpath("./*[contains(@epub:type, 'subtitle')]"):
+					node.remove()
+
+			# Else, if the first child of the <hgroup> is a title, then also discard <hgroup> subtitles
+			# Note the concat() so that matching for `title` doesn't match `subtitle`
+			elif hgroup_element.xpath("./*[1][contains(concat(' ', @epub:type, ' '), ' title ')]"):
+				for node in hgroup_element.xpath("./*[contains(@epub:type, 'subtitle')]"):
+					node.remove()
+
+		# Then after processing <hgroup>, the title becomes the 1st <hgroup> child;
+		# if there is a 2nd <hgroup> child after processing, add a colon and space, then the text of the 2nd <hgroup> child.
+		try:
+			title = regex.sub(r"\s+", " ", hgroup_element.xpath("./*[1]")[0].inner_text().strip())
+		except Exception as ex:
+			raise se.InvalidSeEbookException("Couldn't find title in [xhml]<hgroup>[/]") from ex
+
+		subtitle = hgroup_element.xpath("./*[2]")
+		if subtitle:
+			subtitle_text = subtitle[0].inner_text().strip()
+			title += f": {subtitle_text}"
 
 	else:
-		# No <h#> elements found. Try to get the title from the epub:type of the top-level <section> or <article>
-		top_level_wrappers = soup.select("body > section, body > article")
+		# No hgroups, so try to find the first h# element. The title becomes that element's inner text.
+		h_elements = dom.xpath("//*[re:test(name(), '^h[1-6]')][1]")
 
-		if top_level_wrappers:
-			top_level_wrapper = top_level_wrappers[0]
+		if h_elements:
+			h_element = h_elements[0]
 
-			# Only guess the title if there is a single value for epub:type
-			if top_level_wrapper.has_attr("epub:type") and " " not in top_level_wrapper["epub:type"]:
-				title = titlecase(top_level_wrapper["epub:type"])
+			# Strip any endnote references first
+			for node in h_element.xpath("//*[contains(@epub:type, 'noteref')]"):
+				node.remove()
 
-	return title.strip()
+			title = h_element.inner_text().strip()
+
+		else:
+			# No <h#> elements found. Try to get the title from the epub:type of the deepest <section> or <article> that has no <section> or <article> siblings. (Note the parenthesis
+			# to match against `last()`).
+			# This is to catch cases of <section>s nested for recomposability, so that we don't get "Part 1" instead of "Epigraph" (of part 1)
+			top_level_wrappers = dom.xpath("(/html/body//*[name() = 'section' or name() = 'article' and count(preceding-sibling::*) + count(following-sibling::*) = 0])[last()]")
+
+			if top_level_wrappers:
+				top_level_wrapper = top_level_wrappers[0]
+
+				# Only guess the title if there is a single value for epub:type
+				if top_level_wrapper.attribute("epub:type"):
+					# Get the first non-namespaced value as the title
+					for value in top_level_wrapper.attribute("epub:type").split(" "):
+						if ":" not in value:
+							title = titlecase(value.replace("-", " "))
+							break
+
+	# Remove odd spaces and word joiners
+	title = regex.sub(fr"[{se.NO_BREAK_SPACE}]", " ", title)
+	title = regex.sub(fr"[{se.WORD_JOINER}{se.ZERO_WIDTH_SPACE}]", "", title)
+
+	# Collapse spaces possibly introduced by white-space nodes
+	# This matches all white space EXCEPT hair space; note the double negation with uppercase \S
+	# See https://stackoverflow.com/questions/3548949/how-can-i-exclude-some-characters-from-a-class
+	title = regex.sub(fr"[^\S{se.HAIR_SPACE}]+", " ", title)
+
+	return title
