@@ -105,7 +105,8 @@ def _color_to_alpha(image: Image, color=None) -> Image:
 	return new_image
 
 # Note: We can't type hint driver, because we conditionally import selenium for performance reasons
-def render_mathml_to_png(driver, mathml: str, output_filename: Path, output_filename_2x: Path) -> None:
+# iBooks srcset bug: Temporarily ignore this line in pylint
+def render_mathml_to_png(driver, mathml: str, output_filename: Path, output_filename_2x: Path) -> None: # pylint: disable=unused-argument
 	"""
 	Render a string of MathML into a transparent PNG file.
 
@@ -119,9 +120,10 @@ def render_mathml_to_png(driver, mathml: str, output_filename: Path, output_file
 	None.
 	"""
 
-	with tempfile.NamedTemporaryFile(mode="w+") as mathml_file:
+	# For some reason, we must use an .xhtml suffix. Without that, some mathml expressions don't render.
+	with tempfile.NamedTemporaryFile(mode="w+", suffix=".xhtml") as mathml_file:
 		with tempfile.NamedTemporaryFile(mode="w+", suffix=".png") as png_file:
-			mathml_file.write(f"<!doctype html><html><head><meta charset=\"utf-8\"><title>MathML fragment</title></head><body>{mathml}</body></html>")
+			mathml_file.write(f"<?xml version=\"1.0\" encoding=\"utf-8\"?><html xmlns=\"http://www.w3.org/1999/xhtml\"><head><meta charset=\"utf-8\"/><title>MathML</title></head><body>{mathml}</body></html>")
 			mathml_file.seek(0)
 
 			driver.get(f"file://{mathml_file.name}")
@@ -132,7 +134,9 @@ def render_mathml_to_png(driver, mathml: str, output_filename: Path, output_file
 			image = Image.open(png_file.name)
 			image = _color_to_alpha(image, (255, 255, 255, 255))
 			image = image.crop(image.getbbox())
-			image.save(output_filename_2x)
+
+			# iBooks srcset bug: once srcset works in iBooks, uncomment this line
+			# image.save(output_filename_2x)
 
 			# Save normal version
 			image = image.resize((image.width // 2, image.height // 2))
@@ -202,8 +206,8 @@ def remove_image_metadata(filename: Path) -> None:
 		# PNG and other image types we expect are lossless so we can use PIL to remove metadata
 		try:
 			image = Image.open(filename)
-		except UnidentifiedImageError:
-			raise se.InvalidFileException(f"Couldn’t identify image type of [path][link=file://{filename.resolve()}]{filename}[/].")
+		except UnidentifiedImageError as ex:
+			raise se.InvalidFileException(f"Couldn’t identify image type of [path][link=file://{filename.resolve()}]{filename}[/].") from ex
 
 		data = list(image.getdata())
 
@@ -264,8 +268,8 @@ def svg_text_to_paths(in_svg: Path, out_svg: Path, remove_style=True) -> None:
 
 	try:
 		xml = etree.fromstring(str.encode(svg_in_raw))
-	except:
-		raise se.InvalidXmlException(f"Couldn’t parse SVG file: [path][link={in_svg.resolve()}]{in_svg}[/][/]")
+	except Exception as ex:
+		raise se.InvalidXmlException(f"Couldn’t parse SVG file: [path][link={in_svg.resolve()}]{in_svg}[/][/]") from ex
 
 	svg_ns = "{http://www.w3.org/2000/svg}"
 
