@@ -110,8 +110,8 @@ class SeEpub:
 
 			if "<dc:identifier id=\"uid\">url:https://standardebooks.org/ebooks/" not in self.metadata_xml:
 				raise se.InvalidSeEbookException
-		except:
-			raise se.InvalidSeEbookException(f"Not a Standard Ebooks source directory: [path][link=file://{self.path}]{self.path}[/][/].")
+		except Exception as ex:
+			raise se.InvalidSeEbookException(f"Not a Standard Ebooks source directory: [path][link=file://{self.path}]{self.path}[/][/].") from ex
 
 	@property
 	def repo(self) -> git.Repo:
@@ -122,8 +122,8 @@ class SeEpub:
 		if not self._repo:
 			try:
 				self._repo = git.Repo(self.path)
-			except:
-				raise se.InvalidSeEbookException("Couldn’t access this ebook’s Git repository.")
+			except Exception as ex:
+				raise se.InvalidSeEbookException("Couldn’t access this ebook’s Git repository.") from ex
 
 		return self._repo
 
@@ -262,8 +262,8 @@ class SeEpub:
 			try:
 				with open(self.path / "src" / "epub" / "text" / "endnotes.xhtml") as file:
 					self.__endnotes_soup = BeautifulSoup(file.read(), "html.parser")
-			except:
-				raise se.InvalidFileException(f"Could't open file: [path][link=file://{self.path / 'src' / 'epub' / 'text' / 'endnotes.xhtml'}]{self.path / 'src' / 'epub' / 'text' / 'endnotes.xhtml'}[/][/].")
+			except Exception as ex:
+				raise se.InvalidFileException(f"Could't open file: [path][link=file://{self.path / 'src' / 'epub' / 'text' / 'endnotes.xhtml'}]{self.path / 'src' / 'epub' / 'text' / 'endnotes.xhtml'}[/][/].") from ex
 
 		return self.__endnotes_soup
 
@@ -580,8 +580,8 @@ class SeEpub:
 				file.write(xhtml)
 				file.truncate()
 
-		except Exception:
-			raise se.InvalidSeEbookException(f"Couldn’t open endnotes file: [path][link=file://{endnotes_filename}]{endnotes_filename}[/][/].")
+		except Exception as ex:
+			raise se.InvalidSeEbookException(f"Couldn’t open endnotes file: [path][link=file://{endnotes_filename}]{endnotes_filename}[/][/].") from ex
 
 		with concurrent.futures.ProcessPoolExecutor() as executor:
 			for root, _, filenames in os.walk(source_directory):
@@ -726,8 +726,13 @@ class SeEpub:
 
 				with open(Path(root) / filename, "r", encoding="utf-8") as file:
 					file_contents = file.read()
+
+					if regex.search(r"epub:type=\"[^\"]*?glossary[^\"]*?\"", file_contents):
+						properties += "glossary "
+
 					if "http://www.w3.org/1998/Math/MathML" in file_contents:
 						properties += "mathml "
+
 					if ".svg" in file_contents:
 						properties += "svg "
 
@@ -738,6 +743,10 @@ class SeEpub:
 
 				manifest.append(f"<item href=\"text/{filename}\" id=\"{filename}\" media-type=\"application/xhtml+xml\"{properties}/>")
 
+		# Do we have a glossary search key map?
+		if Path(self.path / "src" / "epub" / "glossary-search-key-map.xml").is_file():
+			manifest.append("<item href=\"glossary-search-key-map.xml\" id=\"glossary-search-key-map.xml\" media-type=\"application/vnd.epub.search-key-map+xml\" properties=\"glossary search-key-map\"/>")
+
 		manifest = natsorted(manifest)
 
 		manifest_xhtml = "<manifest>\n\t<item href=\"toc.xhtml\" id=\"toc.xhtml\" media-type=\"application/xhtml+xml\" properties=\"nav\"/>\n"
@@ -909,7 +918,13 @@ class SeEpub:
 
 		from se.se_epub_generate_toc import generate_toc  # pylint: disable=import-outside-toplevel
 
-		return generate_toc(self)
+		toc_xhtml = generate_toc(self)
+
+		# Word joiners and nbsp don't go in the ToC
+		toc_xhtml = toc_xhtml.replace(se.WORD_JOINER, "")
+		toc_xhtml = toc_xhtml.replace(se.NO_BREAK_SPACE, " ")
+
+		return toc_xhtml
 
 	def generate_endnotes(self) -> Tuple[int, int]:
 		"""
@@ -934,8 +949,8 @@ class SeEpub:
 			try:
 				with open(file_path) as file:
 					soup = BeautifulSoup(file.read(), "lxml")
-			except:
-				raise se.InvalidFileException(f"Couldn’t open file: [path][link=file://{file_path}]{file_path}[/][/].")
+			except Exception as ex:
+				raise se.InvalidFileException(f"Couldn’t open file: [path][link=file://{file_path}]{file_path}[/][/].") from ex
 
 			links = soup.find_all("a")
 			needs_rewrite = False
