@@ -7,6 +7,7 @@ the function is very big and it makes editing easier to put it in a separate fil
 """
 
 import fnmatch
+import filecmp
 import os
 import shutil
 import subprocess
@@ -17,7 +18,6 @@ from pathlib import Path
 from typing import List
 import importlib_resources
 
-from bs4 import BeautifulSoup
 from cairosvg import svg2png
 from natsort import natsorted
 from PIL import Image
@@ -65,8 +65,8 @@ def build(self, run_epubcheck: bool, build_kobo: bool, build_kindle: bool, outpu
 	try:
 		output_directory = output_directory.resolve()
 		output_directory.mkdir(parents=True, exist_ok=True)
-	except Exception:
-		raise se.FileExistsException(f"Couldn’t create output directory: [path][link=file://{output_directory}]{output_directory}[/][/].")
+	except Exception as ex:
+		raise se.FileExistsException(f"Couldn’t create output directory: [path][link=file://{output_directory}]{output_directory}[/][/].") from ex
 
 	# All clear to start building!
 	metadata_xml = self.metadata_xml
@@ -85,8 +85,8 @@ def build(self, run_epubcheck: bool, build_kobo: bool, build_kindle: bool, outpu
 		try:
 			identifier = self.metadata_dom.xpath("//dc:identifier")[0].inner_xml().replace("url:", "")
 			asin = sha1(identifier.encode("utf-8")).hexdigest()
-		except:
-			raise se.InvalidSeEbookException(f"Missing [xml]<dc:identifier>[/] element in [path][link=file://{self.metadata_file_path}]{self.metadata_file_path}[/][/].")
+		except Exception as ex:
+			raise se.InvalidSeEbookException(f"Missing [xml]<dc:identifier>[/] element in [path][link=file://{self.metadata_file_path}]{self.metadata_file_path}[/][/].") from ex
 
 		if not self.metadata_dom.xpath("//dc:title"):
 			raise se.InvalidSeEbookException(f"Missing [xml]<dc:title>[/] element in [path][link=file://{self.metadata_file_path}]{self.metadata_file_path}[/][/].")
@@ -98,8 +98,8 @@ def build(self, run_epubcheck: bool, build_kobo: bool, build_kindle: bool, outpu
 
 		url_author = url_author.rstrip("_")
 
-		epub_output_filename = f"{output_filename}{'.proof' if proof else ''}.epub"
-		epub3_output_filename = f"{output_filename}{'.proof' if proof else ''}.epub3"
+		compatible_epub_output_filename = f"{output_filename}{'.proof' if proof else ''}.epub"
+		epub_output_filename = f"{output_filename}{'.proof' if proof else ''}_advanced.epub"
 		kobo_output_filename = f"{output_filename}{'.proof' if proof else ''}.kepub.epub"
 		kindle_output_filename = f"{output_filename}{'.proof' if proof else ''}.azw3"
 
@@ -107,8 +107,8 @@ def build(self, run_epubcheck: bool, build_kobo: bool, build_kindle: bool, outpu
 		se.quiet_remove(output_directory / f"thumbnail_{asin}_EBOK_portrait.jpg")
 		se.quiet_remove(output_directory / "cover.jpg")
 		se.quiet_remove(output_directory / "cover-thumbnail.jpg")
+		se.quiet_remove(output_directory / compatible_epub_output_filename)
 		se.quiet_remove(output_directory / epub_output_filename)
-		se.quiet_remove(output_directory / epub3_output_filename)
 		se.quiet_remove(output_directory / kobo_output_filename)
 		se.quiet_remove(output_directory / kindle_output_filename)
 
@@ -146,7 +146,7 @@ def build(self, run_epubcheck: bool, build_kobo: bool, build_kindle: bool, outpu
 				file.truncate()
 
 		# Output the pure epub3 file
-		se.epub.write_epub(work_epub_root_directory, output_directory / epub3_output_filename)
+		se.epub.write_epub(work_epub_root_directory, output_directory / epub_output_filename)
 
 		# Now add epub2 compatibility.
 
@@ -176,8 +176,8 @@ def build(self, run_epubcheck: bool, build_kobo: bool, build_kindle: bool, outpu
 					file.truncate()
 
 		# Now get a list of original selectors
-		# Remove @supports(){}
-		total_css = regex.sub(r"@supports.+?{(.+?)}\s*}", "\\1}", total_css, flags=regex.DOTALL)
+		# Remove @supports and @media queries
+		total_css = regex.sub(r"@\s*(?:supports|media).+?{(.+?)}\s*}", r"\1}", total_css, flags=regex.DOTALL)
 
 		# Remove CSS rules
 		total_css = regex.sub(r"{[^}]+}", "", total_css)
@@ -333,9 +333,7 @@ def build(self, run_epubcheck: bool, build_kobo: bool, build_kindle: bool, outpu
 
 		# Massage image references in content.opf
 		metadata_xml = metadata_xml.replace("cover.svg", "cover.jpg")
-		metadata_xml = metadata_xml.replace(".svg", ".png")
 		metadata_xml = metadata_xml.replace("id=\"cover.jpg\" media-type=\"image/svg+xml\"", "id=\"cover.jpg\" media-type=\"image/jpeg\"")
-		metadata_xml = metadata_xml.replace("image/svg+xml", "image/png")
 		metadata_xml = regex.sub(r" properties=\"([^\"]*?)svg([^\"]*?)\"", r''' properties="\1\2"''', metadata_xml) # We may also have the `mathml` property
 		metadata_xml = regex.sub(r" properties=\"([^\s]*?)\s\"", r''' properties="\1"''', metadata_xml) # Clean up trailing white space in property attributes introduced by the above line
 		metadata_xml = regex.sub(r" properties=\"\s*\"", "", metadata_xml) # Remove any now-empty property attributes
@@ -357,8 +355,20 @@ def build(self, run_epubcheck: bool, build_kobo: bool, build_kindle: bool, outpu
 				filename = Path(root) / filename_string
 
 				if filename.suffix == ".svg":
+					add_stroke = False
+
+					if filename.name == "logo.svg":
+						# If we're adding stroke to the logo, make sure it's our premade logo and not a 3rd party logo file.
+						# 3rd party logos will get mangled by our naive regexes.
+						with importlib_resources.path("se.data.templates", "logo.svg") as logo_file_path:
+							if filecmp.cmp(logo_file_path, filename):
+								add_stroke = True
+
+					if filename.name == "titlepage.svg":
+						add_stroke = True
+
 					# For night mode compatibility, give the titlepage a 1px white stroke attribute
-					if filename.name in("titlepage.svg", "logo.svg"):
+					if add_stroke:
 						with open(filename, "r+", encoding="utf-8") as file:
 							svg = file.read()
 							paths = svg
@@ -406,7 +416,20 @@ def build(self, run_epubcheck: bool, build_kobo: bool, build_kindle: bool, outpu
 
 					# Convert SVGs to PNGs at 2x resolution
 					# Path arguments must be cast to string
-					svg2png(url=str(filename), write_to=str(filename.parent / (str(filename.stem) + ".png")), scale=2)
+					svg2png(url=str(filename), write_to=str(filename.parent / (str(filename.stem) + ".png")))
+
+					# iBooks srcset bug: once srcset works in iBooks, we can uncomment this line
+					# svg2png(url=str(filename), write_to=str(filename.parent / (str(filename.stem) + "-2x.png")), scale=2)
+
+					# Add the new PNGs to the manifest
+					# iBooks srcset bug: once srcset works in iBooks, we can use this line instead of the one below it
+					# metadata_xml = metadata_xml.replace("<manifest>", f"""<manifest><item href="images/{filename.stem}.png" id="{filename.stem}.png" media-type="image/png"/><item href="images/{filename.stem}-2x.png" id="{filename.stem}-2x.png" media-type="image/png"/>""")
+					metadata_xml = metadata_xml.replace("<manifest>", f"""<manifest><item href="images/{filename.stem}.png" id="{filename.stem}.png" media-type="image/png"/>""")
+
+					# Remove the SVG from the manifest
+					metadata_xml = regex.sub(fr"""<item href="images/{filename.name}" id="[^"]+?" media-type="image/svg\+xml"/>""", "", metadata_xml)
+
+					# Remove the SVG
 					(filename).unlink()
 
 				if filename.suffix == ".xhtml":
@@ -460,7 +483,7 @@ def build(self, run_epubcheck: bool, build_kobo: bool, build_kindle: bool, outpu
 							landmarks_xhtml = regex.sub(r" role=\"doc-.*?\"", "", landmarks_xhtml[0])
 							processed_xhtml = regex.sub(r"<nav epub:type=\"landmarks\">.*?</nav>", landmarks_xhtml, processed_xhtml, flags=regex.DOTALL)
 
-						# But, remove ARIA roles we added to h# tags, because tyically those roles are for sectioning content.
+						# But, remove ARIA roles we added to h# tags, because typically those roles are for sectioning content.
 						# For example, we might have an h2 that is both a title and dedication. But ARIA can't handle it being a dedication.
 						# See The Man Who Was Thursday by G K Chesterton
 						processed_xhtml = regex.sub(r"(<h[1-6] [^>]*) role=\".*?\">", "\\1>", processed_xhtml)
@@ -470,7 +493,10 @@ def build(self, run_epubcheck: bool, build_kobo: bool, build_kindle: bool, outpu
 
 						# We converted svgs to pngs, so replace references
 						processed_xhtml = processed_xhtml.replace("cover.svg", "cover.jpg")
-						processed_xhtml = processed_xhtml.replace(".svg", ".png")
+
+						# iBooks srcset bug: once srcset works in iBooks, we can use this line instead of the one below it
+						# processed_xhtml = regex.sub(r"src=\"([^\"]+?)\.svg\"", "src=\"\\1.png\" srcset=\"\\1-2x.png 2x, \\1.png 1x\"", processed_xhtml)
+						processed_xhtml = regex.sub(r"src=\"([^\"]+?)\.svg\"", "src=\"\\1.png\"", processed_xhtml)
 
 						# To get popup footnotes in iBooks, we have to change epub:endnote to epub:footnote.
 						# Remember to get our custom style selectors too.
@@ -603,8 +629,7 @@ def build(self, run_epubcheck: bool, build_kobo: bool, build_kindle: bool, outpu
 						css = file.read()
 						processed_css = css
 
-						processed_css = regex.sub(r"(page\-break\-(before|after|inside)\s*:\s*(.+))", "\\1\n\t-webkit-column-break-\\2: \\3 /* For Readium */", processed_css)
-						processed_css = regex.sub(r"^\s*hyphens\s*:\s*(.+)", "\thyphens: \\1\n\tadobe-hyphenate: \\1\n\t-webkit-hyphens: \\1\n\t-epub-hyphens: \\1\n\t-moz-hyphens: \\1", processed_css, flags=regex.MULTILINE)
+						processed_css = regex.sub(r"^\s*hyphens\s*:\s*(.+)", "\thyphens: \\1\n\tadobe-hyphenate: \\1\n\t-webkit-hyphens: \\1\n\t-moz-hyphens: \\1", processed_css, flags=regex.MULTILINE)
 						processed_css = regex.sub(r"^\s*hyphens\s*:\s*none;", "\thyphens: none;\n\tadobe-text-layout: optimizeSpeed; /* For Nook */", processed_css, flags=regex.MULTILINE)
 
 						if processed_css != css:
@@ -613,7 +638,7 @@ def build(self, run_epubcheck: bool, build_kobo: bool, build_kindle: bool, outpu
 							file.truncate()
 
 		# Sort out MathML compatibility
-		has_mathml = "mathml" in metadata_xml
+		has_mathml = len(self.metadata_dom.xpath("/package/manifest/*[contains(@properties, 'mathml')]")) > 0
 		if has_mathml:
 			# We import this late because we don't want to load selenium if we're not going to use it!
 			from se import browser # pylint: disable=import-outside-toplevel
@@ -671,7 +696,9 @@ def build(self, run_epubcheck: bool, build_kobo: bool, build_kindle: bool, outpu
 											# Failure! Abandon all hope, and use Firefox to convert the MathML to PNG.
 											se.images.render_mathml_to_png(driver, regex.sub(r"<(/?)m:", "<\\1", line), work_epub_root_directory / "epub" / "images" / f"mathml-{mathml_count}.png", work_epub_root_directory / "epub" / "images" / f"mathml-{mathml_count}-2x.png")
 
-											processed_xhtml = processed_xhtml.replace(line, f"<img class=\"mathml epub-type-se-image-color-depth-black-on-transparent\" epub:type=\"se:image.color-depth.black-on-transparent\" src=\"../images/mathml-{mathml_count}.png\" srcset=\"../images/mathml-{mathml_count}-2x.png 2x, ../images/mathml-{mathml_count}.png 1x\" />")
+											# iBooks srcset bug: once srcset works in iBooks, we can use this line instead of the one below it
+											# processed_xhtml = processed_xhtml.replace(line, f"<img class=\"mathml epub-type-se-image-color-depth-black-on-transparent\" epub:type=\"se:image.color-depth.black-on-transparent\" src=\"../images/mathml-{mathml_count}.png\" srcset=\"../images/mathml-{mathml_count}-2x.png 2x, ../images/mathml-{mathml_count}.png 1x\"/>")
+											processed_xhtml = processed_xhtml.replace(line, f"<img class=\"mathml epub-type-se-image-color-depth-black-on-transparent\" epub:type=\"se:image.color-depth.black-on-transparent\" src=\"../images/mathml-{mathml_count}.png\"/>")
 											mathml_count = mathml_count + 1
 										else:
 											# Success! Replace the MathML with our new string.
@@ -757,15 +784,15 @@ def build(self, run_epubcheck: bool, build_kobo: bool, build_kindle: bool, outpu
 			se.formatting.format_xml_file(filepath)
 
 		# Write the compatible epub
-		se.epub.write_epub(work_epub_root_directory, output_directory / epub_output_filename)
+		se.epub.write_epub(work_epub_root_directory, output_directory / compatible_epub_output_filename)
 
 		if run_epubcheck:
 			# Path arguments must be cast to string for Windows compatibility.
 			with importlib_resources.path("se.data.epubcheck", "epubcheck.jar") as jar_path:
 				try:
-					epubcheck_result = subprocess.run(["java", "-jar", str(jar_path), "--quiet", str(output_directory / epub_output_filename)], stdin=subprocess.PIPE, stdout=subprocess.PIPE, stderr=subprocess.STDOUT, check=False)
+					epubcheck_result = subprocess.run(["java", "-jar", str(jar_path), "--quiet", str(output_directory / compatible_epub_output_filename)], stdin=subprocess.PIPE, stdout=subprocess.PIPE, stderr=subprocess.STDOUT, check=False)
 					epubcheck_result.check_returncode()
-				except subprocess.CalledProcessError:
+				except subprocess.CalledProcessError as ex:
 					output = epubcheck_result.stdout.decode().strip()
 					# Get the epubcheck version to print to the console
 					version_output = subprocess.run(["java", "-jar", str(jar_path), "--version"], stdin=subprocess.PIPE, stdout=subprocess.PIPE, stderr=subprocess.STDOUT, check=False).stdout.decode().strip()
@@ -776,14 +803,21 @@ def build(self, run_epubcheck: bool, build_kobo: bool, build_kindle: bool, outpu
 					split_output = output.split("\n")
 					output = "\n".join(split_output[:-2])
 
-					# Try to linkify files in output if we can find them
-					try:
-						output = regex.sub(r"(ERROR\(.+?\): )(.+?)(\([0-9]+,[0-9]+\))", lambda match: match.group(1) + "[path][link=file://" + str(self.path / "src" / regex.sub(fr"^\..+?\.epub{os.sep}", "", match.group(2))) + "]" + match.group(2) + "[/][/]" + match.group(3), output)
-					except:
-						# If something goes wrong, just pass through the usual output
-						pass
+					# epubcheck 4.2.4 has a bug where the glossary attribute is not allowed in content.opf.
+					# If that's the only error we have, continue as if we passed.
+					# This check can be removed once epubcheck fixes their bug.
+					split_output = output.split("\n")
+					bugged_epubcheck = len(split_output) == 1 and "OPF-015" in split_output[0] and "glossary" in split_output[0]
 
-					raise se.BuildFailedException(f"[bash]epubcheck[/] v{version} failed with:\n{output}")
+					if not bugged_epubcheck:
+						# Try to linkify files in output if we can find them
+						try:
+							output = regex.sub(r"(ERROR\(.+?\): )(.+?)(\([0-9]+,[0-9]+\))", lambda match: match.group(1) + "[path][link=file://" + str(self.path / "src" / regex.sub(fr"^\..+?\.epub{os.sep}", "", match.group(2))) + "]" + match.group(2) + "[/][/]" + match.group(3), output)
+						except:
+							# If something goes wrong, just pass through the usual output
+							pass
+
+						raise se.BuildFailedException(f"[bash]epubcheck[/] v{version} failed with:\n{output}") from ex
 
 		if build_kindle:
 			# There's a bug in Calibre <= 3.48.0 where authors who have more than one MARC relator role
@@ -809,14 +843,14 @@ def build(self, run_epubcheck: bool, build_kobo: bool, build_kindle: bool, outpu
 			with open(work_epub_root_directory / "epub" / toc_filename, "r+", encoding="utf-8") as file:
 				xhtml = file.read()
 
-				soup = BeautifulSoup(xhtml, "lxml")
+				dom = se.formatting.EasyXhtmlTree(xhtml)
 
-				for match in soup.select("ol > li > ol > li > ol"):
-					match.parent.insert_after(match)
-					match.unwrap()
+				for node in dom.xpath("//ol/li/ol/li/ol"):
+					node.lxml_element.getparent().addnext(node.lxml_element)
+					node.unwrap()
 
 				file.seek(0)
-				file.write(str(soup))
+				file.write(dom.tostring())
 				file.truncate()
 
 			# Rebuild the NCX
@@ -850,8 +884,8 @@ def build(self, run_epubcheck: bool, build_kobo: bool, build_kindle: bool, outpu
 						# First, fixup the reference link for this endnote
 						try:
 							ref_link = etree.tostring(note.xpath("p[last()]/a[last()]")[0], encoding="unicode", pretty_print=True, with_tail=False).replace(" xmlns:epub=\"http://www.idpf.org/2007/ops\"", "").strip()
-						except Exception:
-							raise se.InvalidXhtmlException(f"Can’t find ref link for [url]#{note_id}[/].")
+						except Exception as ex:
+							raise se.InvalidXhtmlException(f"Can’t find ref link for [url]#{note_id}[/].") from ex
 
 						new_ref_link = regex.sub(r">.*?</a>", ">" + note_number + "</a>.", ref_link)
 
@@ -928,14 +962,14 @@ def build(self, run_epubcheck: bool, build_kobo: bool, build_kindle: bool, outpu
 				se.typography.hyphenate_file(filepath, None, True)
 
 			# Build an epub file we can send to Calibre
-			se.epub.write_epub(work_epub_root_directory, work_directory / epub_output_filename)
+			se.epub.write_epub(work_epub_root_directory, work_directory / compatible_epub_output_filename)
 
 			# Generate the Kindle file
 			# We place it in the work directory because later we have to update the asin, and the mobi.update_asin() function will write to the final output directory
 			cover_path = work_epub_root_directory / "epub" / self.metadata_dom.xpath("//item[@properties=\"cover-image\"]/@href")[0].replace(".svg", ".jpg")
 
 			# Path arguments must be cast to string for Windows compatibility.
-			return_code = subprocess.run([str(ebook_convert_path), str(work_directory / epub_output_filename), str(work_directory / kindle_output_filename), "--pretty-print", "--no-inline-toc", "--max-toc-links=0", "--prefer-metadata-cover", f"--cover={cover_path}"], stdout=subprocess.DEVNULL, stderr=subprocess.DEVNULL, check=False).returncode
+			return_code = subprocess.run([str(ebook_convert_path), str(work_directory / compatible_epub_output_filename), str(work_directory / kindle_output_filename), "--pretty-print", "--no-inline-toc", "--max-toc-links=0", "--prefer-metadata-cover", f"--cover={cover_path}"], stdout=subprocess.DEVNULL, stderr=subprocess.DEVNULL, check=False).returncode
 
 			if return_code:
 				raise se.InvalidSeEbookException("[bash]ebook-convert[/] failed.")
