@@ -52,12 +52,14 @@ class TocItem:
 	roman = ""
 	title = ""
 	subtitle = ""
+	title_is_ordinal = False
 	lang = ""
 	id = ""
 	epub_type = ""
 	division = BookDivision.NONE
 	place = Position.FRONT
 
+	@property
 	def toc_link(self) -> str:
 		"""
 		Generates the hyperlink for the ToC item.
@@ -70,24 +72,37 @@ class TocItem:
 		"""
 
 		out_string = ""
-		if self.title is None:
-			return ""
+		if not self.title:
+			raise se.InvalidInputException(f"Couldn't find title in': [path][link=file://{self.file_link}[/][/].")
 
-		lang_attr = ""
-		if self.lang:
-			lang_attr = f" xml:lang=\"{self.lang}\""
+		if self.subtitle and self.lang:
+			# test for a foreign language subtitle, and adjust accordingly
+			self.subtitle = f"<span xml:lang=\"{self.lang}\">{self.subtitle}</span>"
 
 		# If the title is entirely Roman numeral, put epub:type within <a>.
-		if regex.search(r"^<span epub:type=\"z3998:roman\">[IVXLC]{1,10}<\/span>$", self.title):
-			if self.subtitle == "":
-				out_string += f"<a href=\"text/{self.file_link}\" epub:type=\"z3998:roman\"{lang_attr}>{self.roman}</a>\n"
+		if regex.search(r"^<span epub:type=\"z3998:roman\">[IVXLC]+<\/span>$", self.title):
+			# title is a pure roman number
+			if self.subtitle == "":  # put the roman flag inside the <a> tag
+				out_string += f"<a href=\"text/{self.file_link}\" epub:type=\"z3998:roman\">{self.roman}</a>\n"
 			else:
-				out_string += f"<a href=\"text/{self.file_link}\"{lang_attr}>{self.title}: {self.subtitle}</a>\n"
-		else:  # Use the subtitle only if we're a Part or Division or Volume
-			if self.subtitle != "" and (self.division in [BookDivision.PART, BookDivision.DIVISION, BookDivision.VOLUME]):
-				out_string += f"<a href=\"text/{self.file_link}\"{lang_attr}>{self.title}: {self.subtitle}</a>\n"
+				out_string += f"<a href=\"text/{self.file_link}\"><span epub:type=\"z3998:roman\">{self.roman}</span>: {self.subtitle}</a>\n"
+		else:
+			# title has text other than a roman numeral
+			if self.subtitle != "" and (self.title_is_ordinal or (self.division in [BookDivision.PART, BookDivision.DIVISION, BookDivision.VOLUME])):
+				# Use the subtitle only if we're a Part or Division or Volume or if title was an ordinal
+				out_string += f"<a href=\"text/{self.file_link}\">{self.title}"
+
+				# Don't append a colon if the ordinal already ends in punctuation, for example  `1.` or `(a)`
+				if not regex.search(r"\p{Punctuation}$", self.title):
+					out_string += ":"
+
+				out_string += f" {self.subtitle}</a>\n"
 			else:
-				out_string += f"<a href=\"text/{self.file_link}\"{lang_attr}>{self.title}</a>\n"
+				# test for a foreign language title, and adjust accordingly
+				if self.lang:
+					out_string += f"<a href=\"text/{self.file_link}\" xml:lang=\"{self.lang}\">{self.title}</a>\n"
+				else:
+					out_string += f"<a href=\"text/{self.file_link}\">{self.title}</a>\n"
 
 		return out_string
 
@@ -113,7 +128,7 @@ class TocItem:
 
 		return out_string
 
-def get_epub_type(soup: BeautifulSoup) -> str:
+def get_file_epub_type(soup: BeautifulSoup) -> str:
 	"""
 	Retrieve the epub_type of this file to see if it's a landmark item.
 
@@ -185,20 +200,23 @@ def add_landmark(soup: BeautifulSoup, textf: str, landmarks: list):
 	None
 	"""
 
-	epub_type = get_epub_type(soup)
+	epub_type = get_file_epub_type(soup)
 	landmark = TocItem()
 	if epub_type != "":
 		landmark.epub_type = epub_type
 		landmark.file_link = textf
 		landmark.place = get_place(soup)
-		title_tag = soup.find("title")
-		if title_tag is not None:
-			landmark.title = title_tag.string
-			if landmark.title is None:
-				# This is a bit desperate, use this only if there's no proper <title> tag in file.
-				landmark.title = landmark.epub_type.capitalize()
+		if epub_type == "halftitlepage":
+			landmark.title = "Half Title"
 		else:
-			landmark.title = landmark.epub_type.capitalize()
+			title_tag = soup.find("title")
+			if title_tag is not None:
+				landmark.title = title_tag.string
+				if landmark.title is None:
+					# This is a bit desperate, use this only if there's no proper <title> tag in file.
+					landmark.title = landmark.epub_type.capitalize()
+			else:
+				landmark.title = landmark.epub_type.capitalize()
 		landmarks.append(landmark)
 
 def process_landmarks(landmarks_list: list, work_type: str, work_title: str):
@@ -214,6 +232,14 @@ def process_landmarks(landmarks_list: list, work_type: str, work_title: str):
 	None
 	"""
 
+	# we don't want frontmatter items to be included once we've started the body items
+	started_body = False
+	for item in landmarks_list:
+		if item.place == Position.BODY:
+			started_body = True
+		if started_body and item.place == Position.FRONT:
+			item.place = Position.NONE
+
 	front_items = [item for item in landmarks_list if item.place == Position.FRONT]
 	body_items = [item for item in landmarks_list if item.place == Position.BODY]
 	back_items = [item for item in landmarks_list if item.place == Position.BACK]
@@ -249,18 +275,18 @@ def process_items(item_list: list) -> str:
 		# Check to see if next item is at same, lower or higher level than us.
 		if next_item.level == this_item.level:  # SIMPLE
 			out_string += "<li>\n"
-			out_string += this_item.toc_link()
+			out_string += this_item.toc_link
 			out_string += "</li>\n"
 
 		if next_item.level > this_item.level:  # PARENT
 			out_string += "<li>\n"
-			out_string += this_item.toc_link()
+			out_string += this_item.toc_link
 			out_string += "<ol>\n"
 			unclosed_ol += 1
 
 		if next_item.level < this_item.level:  # LAST CHILD
 			out_string += "<li>\n"
-			out_string += this_item.toc_link()
+			out_string += this_item.toc_link
 			out_string += "</li>\n"  # Close off this item.
 			torepeat = this_item.level - next_item.level
 			if torepeat > 0 and unclosed_ol > 0:
@@ -371,9 +397,13 @@ def process_headings(soup: BeautifulSoup, textf: str, toc_list: list, nest_under
 	None
 	"""
 
-	# Find all the h1, h2 etc headings.
-	heads = soup.find_all(["h1", "h2", "h3", "h4", "h5", "h6"])
+	place = get_place(soup)
+	is_toplevel = True
 
+	# Find all the hgroups and h1, h2 etc headings.
+	heads = soup.find_all(["hgroup", "h1", "h2", "h3", "h4", "h5", "h6"])
+
+	# special treatment where we can't find any header or hgroups
 	if not heads:  # May be a dedication or an epigraph, with no heading tag.
 		if single_file and nest_under_halftitle:
 			# There's a halftitle, but only this one content file with no subsections,
@@ -401,13 +431,16 @@ def process_headings(soup: BeautifulSoup, textf: str, toc_list: list, nest_under
 		toc_list.append(special_item)
 		return
 
-	place = get_place(soup)
-	is_toplevel = True
 	for heading in heads:
+		# don't process a heading separately if it's within a hgroup
+		if heading.parent.name == "hgroup":
+			continue  # skip it
 		if place == Position.BODY and single_file:
-			toc_item = process_heading(heading, textf, is_toplevel, True)
+			toc_item = process_a_heading(heading, textf, is_toplevel, True)
 		else:
-			toc_item = process_heading(heading, textf, is_toplevel, False)
+			# if it's not a bodymatter item we don't care about whether it's single_file
+			toc_item = process_a_heading(heading, textf, is_toplevel, False)
+
 		# Tricky check to see if we want to include the item because there's a halftitle
 		# but only a single content file with no subsidiary sections.
 		if is_toplevel and single_file and nest_under_halftitle and len(heads) == 1:
@@ -417,12 +450,12 @@ def process_headings(soup: BeautifulSoup, textf: str, toc_list: list, nest_under
 		is_toplevel = False
 		toc_list.append(toc_item)
 
-def process_heading(heading: BeautifulSoup, textf: str, is_toplevel: bool, single_file: bool) -> TocItem:
+def process_a_heading(soup: BeautifulSoup, textf: str, is_toplevel: bool, single_file: bool) -> TocItem:
 	"""
-	Generate and return a TocItem from this heading.
+	Generate and return a single TocItem from this heading.
 
 	INPUTS:
-	heading: a BeautifulSoup tag representing a heading tag
+	soup: a BeautifulSoup tag representing a heading
 	text: the path to the file
 	is_toplevel: is this heading at the top-most level in the file?
 	single_file: is there only one content file in the production (like some Poetry volumes)?
@@ -432,20 +465,20 @@ def process_heading(heading: BeautifulSoup, textf: str, is_toplevel: bool, singl
 	"""
 
 	toc_item = TocItem()
-	parent_sections = heading.find_parents(["section", "article"])
+	parent_sections = soup.find_parents(["section", "article"])
 	if parent_sections:
 		toc_item.level = len(parent_sections)
 	else:
 		toc_item.level = 1
 
 	try:
-		toc_item.division = get_book_division(heading)
-	except se.InvalidInputException:
-		raise se.InvalidInputException(f"Couldn’t identify parent section in file: [path][link=file://{textf}]{textf}[/][/].")
+		toc_item.division = get_book_division(soup)
+	except se.InvalidInputException as ex:
+		raise se.InvalidInputException(f"Couldn’t identify parent section in file: [path][link=file://{textf}]{textf}[/][/].") from ex
 
 	# This stops the first heading in a file getting an anchor id, we don't generally want that.
 	# The exceptions are things like poems within a single-file volume.
-	toc_item.id = get_parent_id(heading) # pylint: disable=invalid-name
+	toc_item.id = get_parent_id(soup)  # pylint: disable=invalid-name
 	if toc_item.id == "":
 		toc_item.file_link = textf
 	else:
@@ -456,21 +489,91 @@ def process_heading(heading: BeautifulSoup, textf: str, is_toplevel: bool, singl
 		else:
 			toc_item.file_link = textf
 
-	toc_item.lang = heading.get("xml:lang") or ""
+	toc_item.lang = soup.get("xml:lang") or ""
 
+	epub_type = soup.get("epub:type") or ""
+
+	# it may be an empty header tag eg <h3>, so we pass its parent rather than itself to evaluate the parent's descendants
+	if not epub_type and soup.name in ["h2", "h3", "h4", "h5", "h6"]:
+		parent = soup.parent
+		if parent:
+			evaluate_descendants(parent, toc_item)
+		else:  # shouldn't ever happen, but... just in case, raise an error
+			raise se.InvalidInputException(f"Header without parent in file: [path][link=file://{textf}]{textf}[/][/].")
+		return toc_item
 	# A heading may include z3998:roman directly,
 	# eg <h5 epub:type="title z3998:roman">II</h5>.
-	attribs = heading.get("epub:type") or ""
-
-	if "z3998:roman" in attribs:
-		toc_item.roman = extract_strings(heading)
+	if "z3998:roman" in epub_type:
+		toc_item.roman = extract_strings(soup)
 		toc_item.title = f"<span epub:type=\"z3998:roman\">{toc_item.roman}</span>"
 		return toc_item
+	if "ordinal" in epub_type:  # but not a roman numeral (eg in Nietzche's Beyond Good and Evil)
+		toc_item.title = extract_strings(soup)
+		toc_item.title_is_ordinal = True
+		return toc_item
+	# may be the halftitle page with a subtitle, so we need to burrow down
+	if ("fulltitle" in epub_type) and (soup.name == "hgroup"):
+		evaluate_descendants(soup, toc_item)
+		return toc_item
+	# or it may be a straightforward one-level title eg: <h2 epub:type="title">Imprint</h2>
+	if "title" in epub_type:
+		toc_item.title = extract_strings(soup)
+		return toc_item
 
-	process_heading_contents(heading, toc_item)
+	# otherwise, burrow down into its structure to get the info
+	evaluate_descendants(soup, toc_item)
 
 	return toc_item
 
+def evaluate_descendants(soup, toc_item):
+	"""
+	Burrow down into a hgroup structure to qualify the ToC item
+
+	INPUTS:
+	soup: a Beautiful Soup object representing a hgroup
+
+	OUTPUTS:
+	toc_item: qualified ToC item
+	"""
+	for child in soup.children:  # we expect these to be h2, h3, h4 etc
+		if isinstance(child, Tag):
+			if not toc_item.lang:
+				toc_item.lang = child.get("xml:lang") or ""
+			epub_type = child.get("epub:type") or ""
+			if not epub_type and child.name in ["h2", "h3", "h4", "h5", "h6"]:
+				# should be a label/ordinal grouping
+				child_strings = extract_strings(child)
+				if "label" in child_strings and "ordinal" in child_strings:  # quick test
+					toc_item.title_is_ordinal = True
+					# strip label
+					child_strings = regex.sub(r"<span epub:type=\"label\">(.*?)</span>", " \\1 ", child_strings)
+					# adjust roman
+					child_strings = regex.sub(r"ordinal z3998:roman", "z3998:roman", child_strings)
+					child_strings = regex.sub(r"z3998:roman ordinal", "z3998:roman", child_strings)
+					# remove ordinal
+					child_strings = regex.sub(r"<span epub:type=\"ordinal\">(.*?)</span>", " \\1 ", child_strings)
+					# remove extra spaces
+					child_strings = regex.sub(r"[ ]{2,}", " ", child_strings)
+					toc_item.title = child_strings.strip()
+				continue  # skip the following
+			if "z3998:roman" in epub_type:
+				toc_item.roman = extract_strings(child)
+				if not toc_item.title:
+					toc_item.title = f"<span epub:type=\"z3998:roman\">{toc_item.roman}</span>"
+			elif "ordinal" in epub_type:  # but not a roman numeral or a labelled item, cases caught caught above
+				if not toc_item.title:
+					toc_item.title = extract_strings(child)
+					toc_item.title_is_ordinal = True
+			if "subtitle" in epub_type:
+				toc_item.subtitle = extract_strings(child)
+			else:
+				if "title" in epub_type:  # this allows for 'fulltitle' to work here, too
+					if toc_item.title or toc_item.roman or toc_item.title_is_ordinal:  # if title already filled, must be a subtitle
+						toc_item.subtitle = extract_strings(child)
+					else:
+						toc_item.title = extract_strings(child)
+	return toc_item
+
 def get_book_division(tag: BeautifulSoup) -> BookDivision:
 	"""
 	Determine the kind of book division. At present only Part and Division
@@ -523,54 +626,6 @@ def strip_notes(text: str) -> str:
 
 	return regex.sub(r'<a .*?epub:type="noteref".*?>.*?<\/a>', "", text)
 
-def process_heading_contents(heading: BeautifulSoup, toc_item: TocItem):
-	"""
-	Run through each item in the heading contents
-	and try to pull out the toc item data.
-
-	INPUTS:
-	heading: a BeautifulSoup object representing a heading tag and its contents
-	toc_item: a ToC item object to be qualified
-
-	OUTPUTS:
-	None
-	"""
-
-	accumulator = ""  # We'll use this to build up the title.
-	for child in heading.contents:
-		if isinstance(child, Tag):
-			epub_type = child.get("epub:type") or ""
-			if epub_type == "":
-				if child.name == "span":  # If it's an otherwise empty <span>, just take contents.
-					toc_item.lang = child.get("xml:lang") or ""
-					accumulator += extract_strings(child)
-				else:  # If it's a tag without epub:type, such as <abbr>, take whole thing, tags and all.
-					accumulator += str(child)
-				continue  # Skip following and go to next child.
-
-			if "z3998:roman" in epub_type:
-				toc_item.roman = extract_strings(child)
-				accumulator += str(child)
-			elif "subtitle" in epub_type:
-				lang = child.get("xml:lang")
-				if lang:
-					toc_item.subtitle = f"<span xml:lang=\"{lang}\">{extract_strings(child)}</span>"
-				else:
-					toc_item.subtitle = extract_strings(child)
-			elif "title" in epub_type:
-				toc_item.lang = child.get("xml:lang") or ""
-				toc_item.title = extract_strings(child)
-			elif "se:" in epub_type:  # Likely to be a semantically tagged italic.
-				accumulator += str(child)  # Include the whole thing, tags and all.
-			else:
-				accumulator += extract_strings(child)
-		else:  # This should be a simple NavigableString.
-			accumulator += str(child)
-
-	if toc_item.title == "":
-		#  Now strip out any linefeeds or tabs we may have encountered.
-		toc_item.title = regex.sub(r"(\n|\t)", "", accumulator)
-
 def process_all_content(file_list: list, text_path: str) -> Tuple[list, list]:
 	"""
 	Analyze the whole content of the project, build and return lists
